#include "xv.h"
#include "strcase.h"
#include <xview/seln.h>
#include <xview/notice.h>

Frame frame_widget;
Frame bookSelFrame;
Frame searchFrame;
Frame gotoFrame;
Textsw textWidget;
Panel_text_item searchPred, searchRange;
Panel_text_item gotoText;
Panel button_panel;
Panel searchPanel, searchPanel2;
Panel gotoPanel, gotoPanel2;
Panel bookSelPanel;
Textsw bookSelText;
Panel text_panel;
Menu file_menu_widget;
Menu edit_menu_widget;
Menu session_menu_widget;
Menu version_menu_widget;

char * s_Goto = "Goto";
char * s_Search = "Search";
char * s_Bookmark = "Bookmark";
char * s_Synchronise = "Synchronise";
char * s_UnSynchronise = "UnSynchronise";


typedef int (*XvCallbackProc)(Panel_item item, Event * event);

static Menu vecToMenu(XvCallbackProc p, char ** vec, int size);

static int openVersionCB(Menu_item m, Menu_item item);
static int openOkCB(Panel_item item, Event * event);
static int search_nextCB(Panel_item item, Event * event);
static int search_prevCB(Panel_item item, Event * event);
static int search_firstCB(Panel_item item, Event * event);
static int edit_menu(Menu_item m, Menu_item item);

static void message(char * mesg)
	{
	if (mesg == NULL)
		xv_set(frame_widget,
			   FRAME_LEFT_FOOTER, "", 
			   NULL);
	else
		xv_set(frame_widget,
			   FRAME_LEFT_FOOTER, mesg, 
			   NULL);
/*	XFlush(XtDisplay(statusLineWidget)); */
	}

static void errorDialog(char * mesg, Xv_Window w)
	{
	notice_prompt(xv_get(w, PANEL_PARENT_PANEL), NULL,
				  NOTICE_MESSAGE_STRINGS, mesg,
				  NULL,
				  NOTICE_BUTTON_YES,      "Ok",
				  NOTICE_TRIGGER,         MS_LEFT,
				  NOTICE_NO_BEEPING,      TRUE,
				  NULL);
	}

int runxview(XviewContext *mcon)
	{
	xv_main_loop(frame_widget);
	return 0;
	}

XviewContext * loadxview(int argc, char **argv)
	{
	XviewContext * mcon;
	char filename[PATHSIZE+1];
	char ** vec;
	int num;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	frame_widget = xv_create(XV_NULL, FRAME, 
							 XV_LABEL, "Bible",
							 FRAME_SHOW_FOOTER,      TRUE,
							 0);
 	button_panel = xv_create(frame_widget, PANEL, 0);

	getVersionListFileName(filename);
	vec = getvector(filename, &num);
	version_menu_widget = vecToMenu((XvCallbackProc)openVersionCB, vec, num);
	freevector(vec);

    file_menu_widget = xv_create(XV_NULL, MENU,
								 MENU_PULLRIGHT_ITEM, "Open", version_menu_widget,
								 MENU_ITEM,
								 MENU_STRING, "Save", NULL,
								 MENU_ITEM,
								 MENU_STRING, "Save As", NULL,
								 MENU_ITEM,
								 MENU_STRING, "Print", NULL,
								 MENU_ITEM,
								 MENU_STRING, "Quit", NULL, 
								 NULL);
    edit_menu_widget = xv_create(XV_NULL, MENU,
								 MENU_NOTIFY_PROC, edit_menu,
								 MENU_STRINGS,
								 s_Goto,
								 s_Search,
								 s_Bookmark,
								 s_Synchronise, 
								 s_UnSynchronise, NULL,
								 NULL);
    session_menu_widget = xv_create(XV_NULL, MENU,
					 MENU_STRINGS,           "Load", "Save", "Save As", "Load Default", NULL,
					 NULL);
    xv_create(button_panel, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "File",
/* 			  PANEL_NOTIFY_PROC,      selected, */
			  PANEL_ITEM_MENU,        file_menu_widget, /* attach menu to button */
			  NULL);
    xv_create(button_panel, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Edit ",
/* 			  PANEL_NOTIFY_PROC,      selected, */
			  PANEL_ITEM_MENU,        edit_menu_widget, /* attach menu to button */
			  NULL);
    xv_create(button_panel, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Session",
/* 			  PANEL_NOTIFY_PROC,      selected, */
			  PANEL_ITEM_MENU,        session_menu_widget, /* attach menu to button */
			  NULL);


    
    window_fit_height(button_panel);

	textWidget = xv_create(frame_widget, TEXTSW, 
							XV_X, 0,
						   TEXTSW_MEMORY_MAXIMUM, TEXTSW_INFINITY,
						   TEXTSW_UPPER_CONTEXT, 0,
							WIN_BELOW, button_panel,
/* 							WIN_COLUMNS, 80,
							WIN_ROWS, 20, */
							0);
/*	xv_set(textWidget, TEXTSW_CONTENTS, contents, 0); */


    window_fit(frame_widget);


	/* --------------------------------
	   Book Selection Dialog
	   -------------------------------- */
	
	
	bookSelFrame = (Frame)xv_create(frame_widget, FRAME_CMD,
									NULL);
	window_fit(xv_get(bookSelFrame, FRAME_CMD_PANEL));
	
    /* Command frames have panels already created by default -- get it */
/* 	xv_set(bookSelFrame, WIN_COLUMNS, 20, NULL); */

	bookSelText = xv_create(bookSelFrame, TEXTSW, 
							XV_X, 0,
							TEXTSW_READ_ONLY, TRUE, 
 							WIN_COLUMNS, 20,
							WIN_ROWS, 10, 
							NULL);
    bookSelPanel = (Panel)xv_create(bookSelFrame, PANEL,
									XV_X, 0,
									WIN_BELOW, bookSelText,
									NULL);
    xv_create(bookSelPanel, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Ok",
 			  PANEL_NOTIFY_PROC,      openOkCB, 
			  NULL);
    xv_create(bookSelPanel, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Help",
/* 			  PANEL_NOTIFY_PROC,      selected, */
			  NULL);
    window_fit(bookSelPanel);
    window_fit(bookSelFrame);


    

	xv_set(bookSelPanel, WIN_WIDTH, WIN_EXTEND_TO_EDGE, NULL);
	/* --------------------------------
	   Search Dialog
	   -------------------------------- */
	
	
	searchFrame = (Frame)xv_create(frame_widget, FRAME_CMD,
									NULL);
	
    /* Command frames have panels already created by default -- get it */
    searchPanel = (Panel)xv_get(searchFrame, FRAME_CMD_PANEL);
	xv_set(searchPanel,
		   PANEL_LAYOUT, PANEL_VERTICAL,
		   NULL);

/* 	xv_set(searchFrame, WIN_COLUMNS, 20, NULL); */

	searchPred = xv_create(searchPanel, PANEL_TEXT,
						   PANEL_LABEL_STRING, "Search Predicate",
						   PANEL_VALUE_DISPLAY_LENGTH, 20,
						   0);
	searchRange = xv_create(searchPanel, PANEL_TEXT,
							PANEL_LABEL_STRING, "Range                  ",
							PANEL_VALUE_DISPLAY_LENGTH, 20,
							0);
    window_fit(searchPanel);
    searchPanel2 = (Panel)xv_create(searchFrame, PANEL,
									XV_X, 0,
									WIN_BELOW, searchPanel,
									NULL);
		{
		Panel_item next, prev, first, last, help;
    next = xv_create(searchPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Next",
 			  PANEL_NOTIFY_PROC, search_nextCB,
			  NULL);
    prev = xv_create(searchPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Prev",
 			  PANEL_NOTIFY_PROC,      search_prevCB, 
			  NULL);
    first = xv_create(searchPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "First",
 			  PANEL_NOTIFY_PROC,      search_firstCB, 
			  NULL);
    last = xv_create(searchPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Last",
/* 			  PANEL_NOTIFY_PROC,      selected, */
			  NULL);
    help = xv_create(searchPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Help",
/* 			  PANEL_NOTIFY_PROC,      selected, */
			  NULL);
	}
    
    window_fit(searchPanel2);

    window_fit(searchFrame);
	xv_set(searchPanel2, WIN_WIDTH, WIN_EXTEND_TO_EDGE, NULL);
	/* --------------------------------
	   -------------------------------- */
	/* --------------------------------
	   Goto Dialog
	   -------------------------------- */
	
	
	gotoFrame = (Frame)xv_create(frame_widget, FRAME_CMD,
								 NULL);
	
    gotoPanel = (Panel)xv_get(gotoFrame, FRAME_CMD_PANEL);
	xv_set(gotoPanel,
		   PANEL_LAYOUT, PANEL_VERTICAL,
		   NULL);

/* 	xv_set(searchFrame, WIN_COLUMNS, 20, NULL); */

	gotoText = xv_create(gotoPanel, PANEL_TEXT,
						 PANEL_LABEL_STRING, "Goto Verse",
						 PANEL_VALUE_DISPLAY_LENGTH, 20,
						 0);
    window_fit(gotoPanel);
    gotoPanel2 = (Panel)xv_create(gotoFrame, PANEL,
								  XV_X, 0,
								  WIN_BELOW, gotoPanel,
								  NULL);
    xv_create(gotoPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Ok",
			  /* 			  PANEL_NOTIFY_PROC,      selected, */
			  NULL);
    xv_create(gotoPanel2, PANEL_BUTTON,
			  PANEL_LABEL_STRING,     "Help",
			  /* 			  PANEL_NOTIFY_PROC,      selected, */
			  NULL);
    
    window_fit(gotoPanel2);

    window_fit(gotoFrame);
	xv_set(gotoPanel2, WIN_WIDTH, WIN_EXTEND_TO_EDGE, NULL);
	/* --------------------------------
	   -------------------------------- */

	mcon = (XviewContext *)malloc(sizeof (XviewContext));
	mcon->context.run = (int (*)(Context *))runxview;
	return mcon;
	}

static Menu vecToMenu(XvCallbackProc p, char ** vec, int size)
	{
	Menu m;
	Menu_item button;

	m = xv_create(XV_NULL, MENU, 
				  MENU_NOTIFY_PROC, p,
				  NULL);
	for (; *vec != NULL; vec++)
		{
		button = xv_create(XV_NULL, MENUITEM,
						   MENU_STRING, strdup(*vec),
						   NULL);
		xv_set(m, MENU_APPEND_ITEM, button, NULL);
		}
	return m;
	}

static int edit_menu(Menu_item m, Menu_item item)
	{
	STRSWITCH((char *)xv_get(item, MENU_STRING));
	STRCASE(s_Goto);
	xv_set(gotoFrame, XV_SHOW, TRUE, NULL);
	STRCASE(s_Search);
	xv_set(searchFrame, XV_SHOW, TRUE, NULL);
	STRENDSWITCH;
    return XV_OK;
	}

static int openGotoCB(Menu_item m, Menu_item item)
	{
	xv_set(gotoFrame, XV_SHOW, TRUE, NULL);
    return XV_OK;
	}

static int openVersionCB(Menu_item m, Menu_item item)
	{
	char filename[PATHSIZE+1];
	char * selection;

	selection = (char *)xv_get(item, MENU_STRING);
	xv_set(bookSelFrame,
		   FRAME_LABEL, selection,
		   NULL);

    printf("%s selected...\n", selection);
	getBookContentsFileName(filename, selection);
	xv_set(bookSelText,
		   TEXTSW_FILE_CONTENTS, filename,
		   TEXTSW_READ_ONLY, TRUE, 
		   NULL);
	xv_set(bookSelFrame, XV_SHOW, TRUE, NULL);
    return XV_OK;
	}

static int openOkCB(Panel_item item, Event * event)
	{
    Seln_holder holder;
    Seln_request *buffer;
    int line_number;

    /*
     * get primary selection
     */
    holder = seln_inquire(SELN_PRIMARY);
    /*
     * ask for the data containing line number of the first
     * character of the selection
     */
	buffer = seln_ask(&holder,
					  SELN_REQ_FAKE_LEVEL, SELN_LEVEL_LINE,
					  SELN_REQ_FIRST_UNIT, 0,
					  NULL);
	/*
	 * determine the window that contains the selection
	 */
	if (seln_holder_same_client(&holder, (char *)bookSelText))
		{
		char ** vec;
		char * trans;
		int num;
		
		line_number = *(long *)(buffer->data + 3 * sizeof(Seln_attribute));
		trans = (char *)xv_get(bookSelFrame, FRAME_LABEL);
		vec = getBookContentsSVec(trans, &num);
		cVersion = loadBookVersion(trans, vec[line_number]);
		xv_set(textWidget,
			   TEXTSW_CONTENTS, cVersion->cBook->data,
			   TEXTSW_READ_ONLY, TRUE, 
			   TEXTSW_FIRST, 0,
/*			   TEXTSW_NOTIFY_LEVEL, TEXTSW_NOTIFY_NONE, */
/*			   TEXTSW_INSERTION_POINT, 1, */
			   NULL);
		freevector(vec);
		}
	}

static void showVerse(Textsw w, Book * book, AbsVerse ver)
	{
	int foo;
	message(NULL);
	printf("Con:%d\n", xv_get(w, TEXTSW_UPPER_CONTEXT));
/*	textsw_normalize_view(w, book->verses[ver]); */
	xv_set(w,
		   TEXTSW_FIRST, book->verses[ver],
		   NULL);
	for (foo = 0; foo < 20; foo++)
		printf(":%d\n", book->verses[foo]);
	for (foo = 0; foo < 70; foo++)
		printf("%d:%d\n", foo, textsw_index_for_file_line(w, foo));
/*	textsw_normalize_view(w, endOfVerse(book, ver)); */
#ifdef NEVER
	//TEMP
	xv_set(w,
		   TEXTSW_FIRST, book->verses[ver],
		   NULL);
/*	xv_set(w,
		   TEXTSW_FIRST, endOfVerse(book, ver),
		   NULL); */
/*	XmTextSetCursorPosition(w, book->verses[ver]);
	XmTextSetCursorPosition(w, endOfVerse(book, ver)); */
#endif // NEVER
	}

static char * searching = "Searching...";

static char * error_open = "Open a book first";

static void searchForward(Pos pos, Xv_Window w)
	{
	int ver;
	char * expression;
	char * srange;
	Range * range;
	ErrorBuf errbuf;
	Bool nBook;
	int eaten;

	if (cVersion == NULL)
		errorDialog(error_open, w);
	else
	{
		message(searching);
		expression = (char *)xv_get(searchPred, PANEL_VALUE);
		srange = (char *)xv_get(searchRange, PANEL_VALUE);
		
		range = parseVerse(srange, &eaten);
		ver = nextSearchVerse(cVersion->cBook, pos) + 1;
		ver = searchNextVersion(cVersion, ver, range, expression, errbuf, NO, &nBook);
		if (0 <= ver)
			{
			if (nBook)
				{
				xv_set(textWidget,
					   TEXTSW_RESET_TO_CONTENTS, 
					   NULL);
				xv_set(textWidget,
					   TEXTSW_CONTENTS, cVersion->cBook->data,
					   TEXTSW_READ_ONLY, TRUE, 
					   NULL);
				}
			showVerse(textWidget, cVersion->cBook, ver);
			message(NULL);
			}
		else
			message(errbuf);
		}
	}

static int search_nextCB(Panel_item item, Event * event)
	{   
	searchForward(xv_get(textWidget, TEXTSW_FIRST), item);
	}

static int i = 0;

static int search_prevCB(Panel_item item, Event * event)
	{   
	xv_set(textWidget,
		   TEXTSW_FIRST_LINE, ++i,
		   NULL);
	}

static int search_firstCB(Panel_item item, Event * event)
	{   
	xv_set(textWidget,
		   TEXTSW_FIRST_LINE, --i,
		   NULL);
	}
